/*
Layered rock texture (cleavage) as used in scene Crossing Border.pov
by Thomas de Groot.
Version January 2019
*/

//+w1600 +h640 +a0.3 +am2 +bm2 +bs8 +wt6

//=======================================================================================
#version 3.8;

global_settings {
  assumed_gamma 1.0
  radiosity {
    pretrace_start 0.08           // start pretrace at this size
    pretrace_end   0.004          // end pretrace at this size
    count 50, 1000                // higher -> higher quality (1..1600) [35]
    nearest_count 10, 5           // higher -> higher quality (1..10) [5]
    error_bound 1                 // higher -> smoother, less accurate [1.8]
    recursion_limit 2             // how much interreflections are calculated (1..5+) [2]
    low_error_factor .3           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    maximum_reuse 0.1							// new parameter 3.7 [0.2]
    brightness 1                  // brightness of radiosity effects (0..1) [1]

    adc_bailout 0.01/2
    normal on                   // take surface normals into account [off]
    media off                    // take media into account [off]
    always_sample off           // turn sampling in final trace off [on]
    //max_sample 1.0              // maximum brightness of samples
  }
}

sky_sphere {
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6, 0.7, 1.0>]
      [0.7 rgb <0.0, 0.1, 0.8>]
    }
  }
}

camera {
  //ultra_wide_angle
  location  <0, 100, -600.0>
  direction 1.5*z
  right     x*image_width/image_height
  look_at   <0.0, 0.0, 0.0>
  angle 90
}

light_source {
  <0, 0, 0>
  color rgb <1, 1, 1>*2
  translate <-1, 2, 0>*1000
}

plane {y, -1 pigment {rgb 0.3}}

//=======================================================================================
//Scale for the landscape:
//------------------------
#declare HF_scale = <800, 200, 800>;

//-----------------------------------------------------------
//-----------------------------------------------------------
//Rock texture building:
//----------------------
#declare P_rocks =
  pigment {
    gradient y 
    color_map {
      [ 0.40  srgb <0.500, 0.499, 0.490>*1.3]
      [ 0.40  srgb <0.570, 0.527, 0.502>*1.3]
      [ 0.80  srgb <0.570, 0.527, 0.502>*1.3]
      [ 0.80  srgb <0.500, 0.499, 0.490>*1.3]
    }
    scale <1, 0.6, 1>*1 //this makes the rocks more layered
  }

#declare N_cleavage =
normal {
  bump_map {
    png "Bump_StackedPaper_seamless.png" gamma 1.0
    interpolate 2
  }
}

#declare N_rocks =
normal {
  average
  normal_map {
    [0.75 granite 1 turbulence 1 lambda 3 bump_size 0.1]
    [0.75 crackle 1 solid bump_size 5 scale 0.05]
    [1.0 N_cleavage 50 scale <1.0, 0.5, 1.0>*100.0]
  }
}

#declare F_rocks =
finish {
  specular 0.1
  roughness 0.005
  diffuse 0.9
}

#declare T_rocks =
texture {
  pigment {
    gradient y
    pigment_map {
      [0.4 P_rocks]
      [0.4 P_rocks]
    }
    warp {turbulence <0, 0.01, 0>}
  }
  normal {N_rocks warp {turbulence <0.0, 0.01, 0.0>}}
  finish {F_rocks}
}

//-----------------------------------------------------------
//-----------------------------------------------------------
// sand texture for the terrain near the water level
// A light colored sand texture (TdG 2001):
//-----------------------------------------
#declare TdG_LightSand = 
texture {      
  pigment {
    granite
    color_map {
         [ 0.000000  srgb <1.000000, 0.816993, 0.479133> ]
         [ 0.049822  srgb <0.939815, 0.699074, 0.518519> ]
         [ 0.096085  srgb <0.991333, 0.730905, 0.383667> ]
         [ 0.149466  srgb <0.990941, 0.863911, 0.446526> ]
         [ 0.416370  srgb <0.925926, 0.739198, 0.407407> ]
         [ 1.000000  srgb <1.000000, 0.971785, 0.729133> ]
    }
    turbulence 5.0
    lambda 6.0
    frequency 6.0
    ramp_wave
    warp {turbulence <5.0, 2.0, 1.0>}
    scale 0.02
  }      
  normal{
    granite 1
    scale 0.001
  }     
  finish {
    ambient 0
    specular 0.1
    roughness 0.001
  }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
// grass pigments from Islay photograph
//-------------------------------------

#declare GrassGreen =
pigment {
  bozo
  pigment_map {
    [0.00 srgb <99, 141, 15>/255]
    [0.20 srgb <102, 142, 17>/255]
    [0.40 srgb <97, 139, 29>/255]
    [0.60 srgb <114, 154, 31>/255]
    [0.80 srgb <102, 138, 12>/255]
    [1.00 srgb <99, 141, 15>/255]
  }
}

#declare GrassYellow =
pigment {
  bozo
  pigment_map {
    [0.00 srgb <153, 124, 58>/255]
    [0.20 srgb <168, 162, 52>/255]
    [0.40 srgb <150, 130, 59>/255]
    [0.60 srgb <168, 153, 52>/255]
    [0.80 srgb <170, 147, 53>/255]
    [1.00 srgb <153, 124, 58>/255]
  }
}

#declare GrassBrown =
pigment {
  bozo
  pigment_map {
    [0.00 srgb <155, 108, 52>/255]
    [0.20 srgb <195, 154, 102>/255]
    [0.40 srgb <158, 112, 53>/255]
    [0.60 srgb <146, 110, 58>/255]
    [0.80 srgb <153, 114, 59>/255]
    [1.00 srgb <155, 108, 52>/255]
  }
}

#declare P_GrassGreen =
pigment {
  granite
  pigment_map {
    [0.00 GrassGreen]
    [0.40 GrassYellow]
    [0.60 GrassGreen]
    [0.95 GrassBrown]
    [1.00 GrassGreen]
  }
  scale 2
}

#declare P_GrassYellow =
pigment {
  granite
  pigment_map {
    [0.00 GrassYellow]
    [0.40 GrassGreen]
    [0.60 GrassYellow]
    [0.95 GrassBrown]
    [1.00 GrassYellow]
  }
  scale 2
}

#declare P_GrassBrown =
pigment {
  granite
  pigment_map {
    [0.00 GrassBrown]
    [0.40 GrassYellow]
    [0.60 GrassBrown]
    [0.95 GrassGreen]
    [1.00 GrassBrown]
  }
  scale 2
}

#declare F_grass =
finish {
  diffuse 0.8
  specular 0.3
  roughness 0.001
}

//-----------------------------------------------------------
//-----------------------------------------------------------
//Texture building
//----------------
#declare T_terrain =
texture {
  slope {<0, -0.9, 0> 0.5, 1.0 altitude <0, 0.0, 0> 0, 1}
  texture_map {
    [0.30 P_GrassGreen normal {granite 0.5 scale 0.001} finish {F_grass}]  //green
    [0.30 P_GrassYellow normal {granite 0.5 scale 0.001} finish {F_grass}] //yellow
    [0.55 P_GrassBrown normal {granite 0.5 scale 0.001} finish {F_grass}]  //pale brown
    [0.70 P_GrassBrown normal {granite 0.5 scale 0.001} finish {F_grass}]  //pale brown
    [0.70 T_rocks scale <1, 1, 1>*2 rotate -1*z]
  }
}

//-----------------------------------------------------------
//-----------------------------------------------------------
#declare Terrain_Pattern =
texture {
  pigment_pattern {gradient y scale <1, HF_scale.y, 1>}
  texture_map {
    [0.040 TdG_LightSand]
    [0.045 T_terrain]
  }
}

//====================================================================================
#declare MyFn = 
function {
  pigment {
    granite 
    color_map { [0 rgb 0] [1 rgb 1] }
    scale 0.5
  } 
}

#declare MyFn2 = 
function {
  pigment {
    bozo 
    color_map { [0 rgb 0] [1 rgb 1] }
    scale 0.1
  } 
}

#declare HF =
intersection {
  height_field {
    function 1000,1000 {MyFn(x,y,z).hf * MyFn2(x,y*2,z).hf}
    translate <-0.5 0.0, -0.5>
    scale HF_scale
  }
  box {
    <-0.5, 0.0, -0.5>, <0.5, 1.0, 0.5>
    scale HF_scale * <0.99, 1, 0.99>
  }
  scale <-1, 0.5, 1>
  rotate 180*y
}

#object {HF
  texture {Terrain_Pattern}
  scale <1, 1, 1>
}

//Water:
box {
  <-0.5, -1.0, -0.5>, <0.5, 0.0, 0.5>
  scale <HF_scale.x+1, 10.0, HF_scale.z+1>
  translate 8.00*y
  pigment {srgbt <0, 0, 1, 0.4>}
}

  
